# -*- coding: latin-1 -*-
#    QUENLIG: Questionnaire en ligne (Online interactive tutorial)
#    Copyright (C) 2005-2006 Thierry EXCOFFIER, Universite Claude Bernard
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    Contact: Thierry.EXCOFFIER@bat710.univ-lyon1.fr
#

from questions import *
from check import *

etc_required = require("/etc/", "On cherche dans <tt>/etc</tt>")
echo_required= require("echo", "On vous dit d'utiliser <tt>echo</tt>")

add(name="intro",
    required=["sh:affiche paramtre", "sh:configurer"],
    question="""En utilisant la commande <tt>echo</tt>&nbsp;:
    Donner la commande affichant la liste des fichiers et
    rpertoires contenu dans <tt>/etc</tt> dont le nom
    commence par la lettre 'a'""",
    tests=(
    shell_good("echo /etc/a*"),
    etc_required, echo_required,
    reject("ls",
           """On a pas besoin de <tt>ls</tt> car le
           remplacement du <em>pattern</em> fait
            le mme travail"""),
    reject("find",
           """<tt>find</tt> cherche en profondeur
           ici on veut simplement les fichiers
           qui sont directement dans <tt>/etc</tt>"""),
    ),
    )

require_star = require("*",
                       """Il faut utiliser un <em>pattern</em>
                       reprsentant une chaine de caractres
                       quelconques de longueur quelconque.
                       On peut en mettre un pour plusieurs.""")

reject_dot_slash = reject("./",
                           """Pourquoi indiquer le rpertoire courant avec
                           <tt>./</tt>&nbsp;?,
                           par dfaut, les noms des fichiers sont relatifs
                           par rapport  lui.""")

star_indice = """Le symbole reprsentant une chaine de quelconque
    est <tt>*</tt>"""

add(name="tout",
    required=['intro'],
    question="""En utilisant la commande <tt>echo</tt>&nbsp;:
    Donner la commande affichant la liste des fichiers et
    rpertoires contenus dans le rpertoire courant""",
    tests=(
    shell_good("echo *"),
    echo_required,
    require_star,
    reject_dot_slash,
    ),
    indices=(star_indice,
             ),
   )

add(name="dbut fin",
    question="""En utilisant la commande <tt>echo</tt>&nbsp;:
    Donner la commande affichant la liste des fichiers et
    rpertoires contenu dans <tt>/etc</tt> qui commencent
    par une majuscule et qui se terminent par un chiffre.
    """,
    tests=(
    shell_good("echo /etc/[A-Z]*[0-9]"),
    etc_required, echo_required,
    reject("[1-9]", "Zro est un chiffre&nbsp;!"),
    require("[0-9]", "On cherche un chiffre"),
    ),
   )

add(name="0 au milieu",
    question="""En utilisant la commande <tt>echo</tt>&nbsp;:
    Donner la commande affichant la liste des fichiers et
    rpertoires contenus dans <tt>/etc</tt> qui contiennent
    le chiffre '0' n'importe ou dans leur nom.
    """,
    tests=(
    shell_good("echo /etc/*0*"),
    shell_good("echo /etc/*[0]*", "<tt>echo /etc/*0*</tt> tait plus simple"),
    shell_bad("echo /etc/*0",
              """Cela n'affiche que les fichiers dont le nom ce termine par
              un caractre <tt>0</tt>"""),
    shell_bad("echo /etc/0*",
              """Cela n'affiche que les fichiers dont le nom commence par
              un caractre <tt>0</tt>"""),
              
    etc_required, echo_required,
    require("0", "Ne confondez pas zro avec la lettre 'o'"),    
    reject("[0]",
           "Pourquoi mettre <tt>[0]</tt> alors que <tt>0</tt> suffit&nbsp;?"),
    ),
   )

add(name="chemin",
    question="""Dans '/etc' il y a des rpertoires dont le nom commence
    par 'rc', ces rpertoires contiennent des fichiers
    dont le nom commence par 'S1'.
    <p>
    Utilisez la commande <tt>echo</tt> et le <em>globbing</em>
    pour afficher la liste de ces fichiers.
    """,
    tests=(
    shell_good("echo /etc/rc*/S1*"),
    etc_required, echo_required,
    require("/rc", "Le nom du premier commence par <tt>rc</tt>"),    
    require("/S1", "Le nom du deuxime commence par <tt>S1</tt>"),    
    ),
    )

add(name="sous rpertoires",
    question="""En utilisant la commande <tt>echo</tt>.
    Donner la commande affichant la liste des rpertoires
    contenus dans <tt>/etc</tt> (<em>pas les fichiers,
    seulement les rpertoires</em>).
    """,
    tests=(
    shell_good( ("echo /etc/*/.", "echo /etc/*/") ),
    etc_required, echo_required,
    shell_bad("echo /etc/*",
              "Cela affiche aussi les fichiers dans <tt>/etc</tt>"),
    ),
    indices=("Seul un rpertoire peut contenir un fichier nomm '.'",
             """On peut mettre une toile entre deux <tt>/</tt> elle
             sera alors remplace par tous les noms de rpertoires.""",
             ),
   )

add(name="fini par tilde",
    required=['intro'],
    question="""En utilisant la commande <tt>echo</tt>&nbsp;:
    Donner la commande affichant la liste des fichiers et
    rpertoires contenus dans le rpertoire courant dont le nom se
    termine par <tt>~</tt>""",
    tests=(
    shell_good("echo *~"),
    echo_required,
    require_star,
    reject_dot_slash,
    ),
   )

add(name=".c et .h",
    required=['fini par tilde'],
    question="""En utilisant la commande <tt>echo</tt>&nbsp;:
    Donner la commande affichant la liste des fichiers et
    rpertoires contenus dans le rpertoire courant dont le nom se
    termine par <tt>.c</tt> ou bien <tt>.h</tt>""",
    tests=(
    shell_good(("echo *.[ch]", "echo *.[hc]")),
    shell_bad(("echo *.c *.h", "echo *.h *.c"),
              """Il y a plus court, avec une seule toile."""),
    reject(('[c]', '[h]'),
           """ quoi cela sert d'avoir une liste contenant
           un seul caractre&nbsp;?
           Autant taper le caractre lui mme."""),
    echo_required,
    require_star,
    reject_dot_slash,
    ),
    indices=(star_indice,
             """Utilisez le pattern dsignant un ensemble de caractres
             pour indiquer qu'aprs le <tt>.</tt> il y a <tt>c</tt>
             ou <tt>h</tt>""",
             """Il faut utiliser les crochets.""",
             ),
   )

add(name="inverse",
    required=["0 au milieu", ".c et .h"],
    question="""En utilisant la commande <tt>echo</tt>.
    Donner la commande affichant la liste des fichiers et
    rpertoires contenus dans <tt>/etc</tt> dont le nom
    contient un caractre qui ne soit PAS&nbsp;:
    <ul>
    <li> -
    <li> .
    <li>lettre minuscule
    <li>lettre majuscule
    <li>numrique
    </ul>
    Conservez l'ordre pour que votre rponse soit accepte.
    """,
    tests=(
    shell_good("echo /etc/*[!-.a-zA-Z0-9]*"),
    echo_required, etc_required,
    shell_bad("echo /etc/[!-.a-zA-Z0-9]",
              """Vous ne trouvez que des noms de fichier ne comportant
              qu'un seul caractre"""),
    shell_bad("echo /etc/*[!-.a-zA-Z0-9]",
              """Vous ne trouverez pas <tt>/etc/=a</tt>
              qui contient un caractre interdit mais ce termine
              par un caractre autoris"""),
    shell_bad("echo /etc/[!-.a-zA-Z0-9]*",
              """Vous ne trouverez pas <tt>/etc/a=</tt>
              qui contient un caractre interdit mais commence
              par un caractre autoris"""),
    reject(("]]", "]["), "N'imbriquez pas les crochets"),
    require("!", "La ngation s'indique avec le <tt>!</tt>"),
    require("a-z", "Et les minuscules&nbsp;?"),
    require("A-Z", "Et les majuscules&nbsp;?"),
    require("0-9", "Et les nombres&nbsp;?"),
    require("-.", "Et le tiret et le point&nbsp;?"),
    require("-.a-zA-Z0-9", "Il faut conserver l'ordre"),
    ),
    good_answer="Si 'echo' vous affiche <tt>/etc/*[!-.a-zA-Z0-9]*</tt> c'est que '/etc' ne contient pas de tel fichier",
   )

add(name="final",
    required=["inverse", "sous rpertoires"],
    question="""En utilisant la commande <tt>echo</tt>.
    affichez tous les fichiers avec l'extension <tt>.h</tt>
    qui sont dans des sous rpertoires <em>directs</em>
    de <tt>/usr/include</tt>""",
    tests=(
    shell_good("echo /usr/include/*/*.h"),
    echo_required,
    reject("/usr/include/*.h",
           """Vous indiquez les <tt>.h</tt> qui sont dans
           <tt>/usr/include</tt>, pas dans ses sous-rpertoires"""),
    require("/usr/include",
            "On vous demande de chercher dans <tt>/usr/include</tt>"),
    ),
    )





